/*
        cstrstream.h -- fast strstream operations
        (C) Copyright 1995  John Webster Small
        All rights reserved
*/

#include "cstrstrm.h"

orcstrstream::orcstrstream(cstr& cs, int packOnDestruct)
    : ostrstream((char *)(const char *)cs,
        int(cs.limit()-1),ios::ate)
    , _packed(0), _pcount(0)
    , _packOnDestruct(packOnDestruct)
    , cs_(cs)
    , csStartLen(cs.len())
{
  assert(cs.limit() && cs.limit() <= size_t(INT_MAX));
}

int orcstrstream::pcount()
{
     if (_packed)
         return _pcount;
     return _pcount = ostrstream::pcount();
}

void orcstrstream::reset()
{
    assert(!_packed);
//    (ostrstream::rdbuf())->freeze(0);
    (void) seekp(0,ios::beg);
}

void orcstrstream::pack()
{
    if (!_packed)  {
        cs_.bufPack(pcount()+csStartLen);
        (void) ostrstream::str();
        ostrstream::clear(ios::failbit | ios::badbit);
        _packed = 1;
    }
}

orcstrstream::~orcstrstream()
{
    if (_packOnDestruct)
        pack();
    else  {
        *this << ends;
        cs_.bufCalc();
    }
}

rcstrstream::rcstrstream(cstr& cs, int packOnDestruct)
    : strstream((char *)(const char *)cs,
        int(cs.limit()-1),ios::ate)
    , _packed(0), _pcount(0)
    , _packOnDestruct(packOnDestruct)
    , cs_(cs)
    , csStartLen(cs.len())
{
  assert(cs.limit() && cs.limit() <= size_t(INT_MAX));
}

void rcstrstream::reset()
{
    assert(!_packed);
//    (strstream::rdbuf())->freeze(0);
    (void) seekg(0,ios::beg);
    (void) seekp(0,ios::beg);
}

int rcstrstream::pcount()
{
    if (_packed)
        return _pcount;
    (void) peek();  // initialize gptr!
    long tmp = tellp() - tellg();
    assert(tmp <= INT_MAX);
    return _pcount = (int) tmp;
}

void rcstrstream::pack()
{
    if (!_packed)  {
        cs_.bufPack(pcount()+csStartLen);
        (void) strstream::str();
        strstream::clear(ios::failbit | ios::badbit);
        _packed = 1;
    }
}

rcstrstream::~rcstrstream()
{
    if (_packOnDestruct)
        pack();
    else  {
        *this << ends;
        cs_.bufCalc();
    }
}
